%* lsaf_getjobprograms                                                                             *;
%*                                                                                                 *;
%* Gets the metadata for the programs for the specified job in the SAS Life Science Analytics      *;
%* Framework repository. The metadata is stored in a SAS data set.                                 *;
%*                                                                                                 *;
%* macrotype LSAF                                                                                  *;
%* since 1.4                                                                                       *;
%* exposure external                                                                               *;
%*                                                                                                 *;
%* History                                                                                         *;
%* 2014-04-22  initial coding.                                                                     *;
%* 2016-08-26  rebrand as LSAF                                                                     *;
%* 2019-02-11  update to 2.1                                                                       *;
%* 2019-02-26  rename from lsaf_getjobtasks to lsaf_getjobprograms                                 *;
%* 2019-11-01  update to handle a blank SAS output file.                                           *;
%* 2021-04-07  update for java code refactor                                                       *;
%* 2021-08-06  Remove header documentation                                                         *;
%*                                                                                                 *;

%macro lsaf_getjobprograms(
   lsaf_path=,
   lsaf_version=,
   sas_dsname=work.lsafGetJobPrograms
   ) /des='Get a list of the programs for a job.';
  
   %if "&sas_dsname" EQ "" %then %let sas_dsname=work.lsafGetJobPrograms;

   %****************************************;
   %* Initialize macro variables           *;
   %****************************************;
   %lsaf_getProductName();

   %global _lsafMsg_
           _lsafRC_
   ;

   %let _lsafRC_ =%str(-999);
   %let _lsafMsg_=%str(The SAS Macro did not execute properly.  Unknown err%str(or).);

   %**********************************************;
   %* Start the datastep to call the java code   *;
   %**********************************************;
   data &sas_dsname (label="Created by &SYSMACRONAME"
                     keep=jobPath jobVersion programPath programVersion
   );
   attrib returnCode      length=8
          message         length=$200
          logMsgPrefix    length=$70
          logMsgPrimary   length=$70
          logMsg          length=$300
          rowCount        length=8

          jobPath         length=$2048     label="Job Path"
          jobVersion      length=$32       label="Job Version"
          programPath     length=$2048     label="Program File Path"
          programVersion  length=$32       label="Program File Version"
   ;

   rowCount=0;

   %***********************************;
   %* Declare the java objects        *;
   %***********************************;
   declare javaobj srv("com/sas/lsaf/macro/execution/job/JobService") ;
   declare javaobj results("com/sas/lsaf/macro/execution/job/SasJobProgramsResult");

   %***********************************;
   %* Attempt the operation           *;
   %***********************************;
   srv.callStaticVoidMethod("getRepositoryJobPrograms", "%bquote(&lsaf_path)", "%bquote(&lsaf_version)", results) ;

   %***********************************;
   %* Retrieve the results            *;
   %***********************************;
   results.callIntMethod("getReturnCode", returnCode);
   results.callStringMethod("getReturnMessage", message);

   %***********************************;
   %* Process the results             *;
   %***********************************;
   logMsgPrefix="NOTE: &_LSAF_PRODUCT_NAME_ Macro: * ";
   if (returnCode ne 0) then
   do;
      logMsgPrimary="ERR%str(OR): &_LSAF_PRODUCT_NAME_ Macro:";
      if "&lsaf_version" NE ""
        then logMsg=" Cannot retrieve programs for job &lsaf_path, version &lsaf_version" ;
        else logMsg=" Cannot retrieve programs for job &lsaf_path" ;
   end;
   else
   do;
      logMsgPrimary=logMsgPrefix;
      logMsg=" Programs were returned for job &lsaf_path" ;
      results.callStringMethod("getJobPath", jobPath);
      results.callStringMethod("getJobVersion", jobVersion);
      results.callIntMethod("getSize", rowCount);
      do row=0 to rowCount - 1;
         results.callStringMethod("getPath", row, programPath);
         results.callStringMethod("getFileVersion", row, programVersion);
         output;
      end;
   end;

   %***********************************;
   %* Print messages in log           *;
   %***********************************;
   put;
   put logMsgPrimary " " logMsg;
   put logMsgPrefix " " %sysfunc(lowcase("&SYSMACRONAME"));
   if (rowCount=0) 
     then put logMsgPrefix " No programs were found.";
     else put logMsgPrefix " " rowCount " records written to data set &sas_dsname";
   put logMsgPrefix " _lsafMsg_= " message ;
   put logMsgPrefix " _lsafRC_= " returnCode ;
   put;

   %***********************************;
   %* Set the macro variables         *;
   %***********************************;
   call symputx("_lsafRC_", returnCode);
   call symputx("_lsafMsg_", message);

   run;

%mend lsaf_getjobprograms;
